@@ -28,10 +28,12 @@ import http.cookies
 import inspect
 import time
 from typing import Any, Dict, List, Tuple, Type, Union, Callable
+
+
 from .logger import get_logger
 
 name = "simple_http_server"
-version = "0.16.2"
+version = "0.16.4"
 
 DEFAULT_ENCODING: str = "UTF-8"
 
@@ -76,20 +78,24 @@ class Session:
     def is_valid(self) -> bool:
         return time.time() - self.last_accessed_time < self.max_inactive_interval
 
+    @abstractmethod
     def get_attribute(self, name: str) -> Any:
-        return None
+        return NotImplemented
 
+    @abstractmethod
     def set_attribute(self, name: str, value: str) -> None:
-        pass
+        return NotImplemented
 
+    @abstractmethod
     def invalidate(self) -> None:
-        pass
+        return NotImplemented
 
 
 class SessionFactory:
 
+    @abstractmethod
     def get_session(self, session_id: str, create: bool = False) -> Session:
-        return None
+        return NotImplemented
 
 
 class Cookies(http.cookies.SimpleCookie):
@@ -98,8 +104,9 @@ class Cookies(http.cookies.SimpleCookie):
 
 class RequestBodyReader:
 
+    @abstractmethod
     async def read(self, n: int = -1) -> bytes:
-        pass
+        return NotImplemented
 
 
 class Request:
@@ -168,8 +175,7 @@ class Request:
 
     @abstractmethod
     def get_session(self, create: bool = False) -> Session:
-        # This is abstract method
-        return None
+        return NotImplemented
 
 
 class MultipartFile:
@@ -393,17 +399,17 @@ class Response:
             for k, v in headers.items():
                 self.add_header(k, v)
 
+    @abstractmethod
     def send_error(self, status_code: int, message: str = ""):
-        """abstruct method"""
-        raise Exception("Abstruct method, you cannot call this method directly.")
+        return NotImplemented
 
+    @abstractmethod
     def send_redirect(self, url: str):
-        """abstruct method"""
-        raise Exception("Abstruct method, you cannot call this method directly.")
+        return NotImplemented
 
+    @abstractmethod
     def send_response(self):
-        """abstruct method"""
-        raise Exception("Abstruct method, you cannot call this method directly.")
+        return NotImplemented
 
 
 class HttpError(Exception):
@@ -463,15 +469,15 @@ class FilterContex:
 
     @property
     def request(self) -> Request:
-        return None
+        return NotImplemented
 
     @property
     def response(self) -> Response:
-        return None
+        return NotImplemented
 
     @abstractmethod
     def do_chain(self):
-        pass
+        return NotImplemented
 
 
 class WebsocketRequest:
@@ -481,6 +487,7 @@ class WebsocketRequest:
         self.__cookies = Cookies()
         self.query_string: str = ""  # Query String
         self.path_values: Dict[str, str] = {}
+        self.reg_groups = ()  # If controller is matched via regexp, then ,all groups are save here
         self.path: str = ""  # Path
         self.__parameters = {}  # Parameters, key-value array, merged by query string and request body if the `Content-Type` in request header is `application/x-www-form-urlencoded` or `multipart/form-data`
         self.__parameter = {}  # Parameters, key-value, if more than one parameters with the same key, only the first one will be stored.
@@ -515,36 +522,43 @@ class WebsocketSession:
 
     @property
     def id(self) -> str:
-        return ""
+        return NotImplemented
 
     @property
     def request(self) -> WebsocketRequest:
-        return None
+        return NotImplemented
 
     @property
     def is_closed(self) -> bool:
-        return False
+        return NotImplemented
 
+    @abstractmethod
     def send(self, message: Union[str, bytes], opcode: int = None, chunk_size: int = 0):
-        pass
+        return NotImplemented
 
+    @abstractmethod
     def send_text(self, message: str, chunk_size: int = 0):
-        pass
+        return NotImplemented
 
+    @abstractmethod
     def send_binary(self, binary: bytes, chunk_size: int = 0):
-        pass
+        return NotImplemented
 
+    @abstractmethod
     def send_file(self, path: str, chunk_size: int = 0):
-        pass
+        return NotImplemented
 
+    @abstractmethod
     def send_pone(self, message: bytes):
-        pass
+        return NotImplemented
 
+    @abstractmethod
     def send_ping(self, message: bytes):
-        pass
+        return NotImplemented
 
+    @abstractmethod
     def close(self, reason: str):
-        pass
+        return NotImplemented
 
 
 class WebsocketCloseReason(str):
@@ -576,6 +590,7 @@ class WebsocketCloseReason(str):
 
 class WebsocketHandler:
 
+    @abstractmethod
     def on_handshake(self, request: WebsocketRequest = None):
         """
         "
@@ -590,6 +605,7 @@ class WebsocketHandler:
         """
         return None
 
+    @abstractmethod
     def on_open(self, session: WebsocketSession = None):
         """
         " 
@@ -598,6 +614,7 @@ class WebsocketHandler:
         """
         pass
 
+    @abstractmethod
     def on_close(self, session: WebsocketSession = None, reason: WebsocketCloseReason = None):
         """
         "
@@ -606,6 +623,7 @@ class WebsocketHandler:
         """
         pass
 
+    @abstractmethod
     def on_ping_message(self, session: WebsocketSession = None, message: bytes = b''):
         """
         "
@@ -614,6 +632,7 @@ class WebsocketHandler:
         """
         session.send_pone(message)
 
+    @abstractmethod
     def on_pong_message(self, session: WebsocketSession = None, message: bytes = ""):
         """
         "
@@ -622,6 +641,7 @@ class WebsocketHandler:
         """
         pass
 
+    @abstractmethod
     def on_text_message(self, session: WebsocketSession = None, message: str = ""):
         """
         "
@@ -630,6 +650,7 @@ class WebsocketHandler:
         """
         pass
 
+    @abstractmethod
     def on_binary_message(self, session: WebsocketSession = None, message: bytes = b''):
         """
         "
@@ -639,15 +660,15 @@ class WebsocketHandler:
         """
         pass
 
+    @abstractmethod
     def on_binary_frame(self, session: WebsocketSession = None, fin: bool = False, frame_payload: bytes = b''):
         """
         "
-        " If you are sending a continuation binary message to server, this will be called every time a frame is 
-        " received, you can consumed all the bytes in this method, e.g. save all bytes to a file. By doing so, 
-        " you should not return and value in this method. 
+        " When server receive a fragmented message, this method will be called every time when a frame is received, 
+        " you can consume all the bytes in this method, e.g. save all bytes to a file.  
         "
-        " If you does not implement this method or return a True in this method, all the bytes will be caced in
-        " memory and be sent to your `on_binary_message` method.
+        " If you does not implement this method or return a True in this method, all the bytes will be cached in
+        " memory and sent to your `on_binary_message` method after all frames are received.
         "
         """
         return True
@@ -683,15 +704,14 @@ class ControllerFunction:
     def __init__(self, url: str = "",
                  regexp: str = "",
                  method: str = "",
-                 ctr_obj: object = None,
                  func: Callable = None) -> None:
         self.__url: str = url
         self.__regexp = regexp
         self.__method: str = method
-        self.singletion: bool = False
+        self.singleton: bool = False
         self.ctr_obj_init_args = None
         self.ctr_obj_init_kwargs = None
-        self.__ctr_obj: object = ctr_obj
+        self.__ctr_obj: object = None
         self.__func: Callable = func
         self.__clz = False
 
@@ -703,7 +723,7 @@ class ControllerFunction:
             assert self.func is not None and (inspect.isfunction(self.func) or inspect.ismethod(self.func))
             return True
         except AssertionError as ae:
-            _logger.warn(f"[{self._url}|{self.regexp}] => {self.func} configurate error: {ae}")
+            _logger.warn(f"[{self.__url}|{self.regexp}] => {self.func} configurate error: {ae}")
             return False
 
     @property
@@ -730,7 +750,7 @@ class ControllerFunction:
 
     @property
     def ctrl_object(self) -> object:
-        if not self.singletion:
+        if not self.singleton:
             obj = self._create_ctrl_obj()
             _logger.debug(f"singleton: create a object -> {obj}")
             return obj
@@ -754,6 +774,66 @@ class ControllerFunction:
         return self.__func
 
 
+class WebsocketHandlerClass:
+
+    def __init__(self, url: str = "",
+                 cls: Callable = None,
+                 regexp: str = "",
+                 singleton: bool = True
+                 ) -> None:
+        self.__url: str = url
+        self.__regexp = regexp
+        self.singleton: bool = singleton
+        self.ctr_obj_init_args = None
+        self.ctr_obj_init_kwargs = None
+        self.__ctr_obj: object = None
+        self.__cls: Callable = cls
+
+    @property
+    def _is_config_ok(self):
+        try:
+            assert self.url or self.regexp, "you should set one of url and regexp"
+            assert not (self.url and self.regexp), "you can only set one of url and regexp, not both"
+            assert self.cls is not None and inspect.isclass(self.cls)
+            return True
+        except AssertionError as ae:
+            _logger.warn(f"[{self.__url}|{self.regexp}] => {self.cls} configurate error: {ae}")
+            return False
+
+    @property
+    def url(self) -> str:
+        return self.__url
+
+    @property
+    def regexp(self) -> str:
+        return self.__regexp
+
+    @property
+    def cls(self) -> str:
+        return self.__cls
+
+    @property
+    def ctrl_object(self) -> object:
+        if not self.singleton:
+            obj = self._create_ctrl_obj()
+            _logger.debug(f"singleton: create a object -> {obj}")
+            return obj
+
+        if self.__ctr_obj is None:
+            self.__ctr_obj = self._create_ctrl_obj()
+            _logger.debug(f"object does not exist, create one -> {self.__ctr_obj} ")
+        else:
+            _logger.debug(f"object[{self.__ctr_obj}] exists, return. ")
+        return self.__ctr_obj
+
+    def _create_ctrl_obj(self) -> object:
+        return _create_object(self.cls, self.ctr_obj_init_args, self.ctr_obj_init_kwargs)
+
+    @ctrl_object.setter
+    def ctrl_object(self, val) -> None:
+        self.__ctr_obj = val
+
+
 _request_mappings: List[ControllerFunction] = []
 _request_clz_mapping: Dict[Any, Tuple[str, Union[str, list]]] = {}
 
@@ -762,7 +842,7 @@ _filters = []
 _ctrls = {}
 _ctrl_singletons = {}
 
-_ws_handlers = {}
+_ws_handlers: List[WebsocketHandlerClass] = []
 
 _error_page = OrderedDict()
 
@@ -851,9 +931,9 @@ def _get_singletion(clz, args, kwargs):
     return _ctrl_singletons[clz]
 
 
-def websocket_handler(endpoint=""):
+def websocket_handler(endpoint: str = "", regexp: str = "", singleton: bool = True) -> Callable:
     def map(ws_class):
-        _ws_handlers[endpoint] = ws_class
+        _ws_handlers.append(WebsocketHandlerClass(url=endpoint, regexp=regexp, cls=ws_class, singleton=singleton))
         return ws_class
 
     return map
@@ -886,7 +966,7 @@ def error_message(*anno_args):
         return map
 
 
-def _get_request_mappings():
+def _get_request_mappings() -> List[ControllerFunction]:
     mappings: List[ControllerFunction] = []
 
     for ctr_fun in _request_mappings:
@@ -923,7 +1003,7 @@ def _get_request_mappings():
         clz = ctr_fun.clz
         if clz is not None and clz in _ctrls:
             singleton, args, kwargs = _ctrls[clz]
-            ctr_fun.singletion = singleton
+            ctr_fun.singleton = singleton
             ctr_fun.ctr_obj_init_args = args
             ctr_fun.ctr_obj_init_kwargs = kwargs
             if singleton:
@@ -940,7 +1020,7 @@ def _get_session_factory() -> SessionFactory:
     return _session_facory
 
 
-def _get_websocket_handlers() -> Dict[str, Type]:
+def _get_websocket_handlers() -> List[WebsocketHandlerClass]:
     return _ws_handlers
 
 
