@@ -24,6 +24,7 @@ SOFTWARE.
 """
 
 
+from abc import abstractmethod
 import json
 import socket
 import os
@@ -43,7 +44,7 @@ from urllib.parse import unquote
 
 from typing import Any, Callable, Dict, List, Tuple
 
-from simple_http_server import ControllerFunction, StaticFile
+from simple_http_server import ControllerFunction, StaticFile, WebsocketHandlerClass
 from .http_protocol_handler import HttpProtocolHandler, SocketServerStreamRequestHandlerWraper
 from .wsgi_request_handler import WSGIRequestHandler
 
@@ -53,7 +54,7 @@ from .logger import get_logger
 _logger = get_logger("simple_http_server.http_server")
 
 
-class RoutingConf:
+class RoutingServer:
 
     HTTP_METHODS = ["OPTIONS", "GET", "HEAD",
                     "POST", "PUT", "DELETE", "TRACE", "CONNECT"]
@@ -74,8 +75,9 @@ class RoutingConf:
         self._res_conf = []
         self.add_res_conf(res_conf)
 
-        self.ws_mapping = OrderedDict()
-        self.ws_path_val_mapping = OrderedDict()
+        self.ws_mapping: Dict[str, ControllerFunction] = OrderedDict()
+        self.ws_path_val_mapping: Dict[str, ControllerFunction] = OrderedDict()
+        self.ws_regx_mapping: Dict[str, ControllerFunction] = OrderedDict()
 
         self.error_page_mapping = {}
 
@@ -177,9 +179,9 @@ class RoutingConf:
             return fun_and_val[0], fun_and_val[1], ()
 
         # regexp
-        func_and_groups = self.__try_get_from_regexp(path, method)
+        func_and_groups = self.__try_get_ctrl_from_regexp(path, method)
         if func_and_groups is None:
-            func_and_groups = self.__try_get_from_regexp(path, "_")
+            func_and_groups = self.__try_get_ctrl_from_regexp(path, "_")
         if func_and_groups is not None:
             return func_and_groups[0], {}, func_and_groups[1]
         # static files
@@ -190,9 +192,9 @@ class RoutingConf:
                 return ControllerFunction(func=static_fun), {}, ()
         return None, {}, ()
 
-    def __try_get_from_regexp(self, path, method):
+    def __try_get_ctrl_from_regexp(self, path, method):
         for regex, ctrl in self.method_regexp_mapping[method].items():
-            m = re.match(regex, path)
+            m = re.match(regex, f"/{path}") or re.match(regex, path)
             _logger.debug(
                 f"regexp::pattern::[{regex}] => path::[{path}] match? {m is not None}")
             if m:
@@ -222,7 +224,8 @@ class RoutingConf:
             regexp = get_path_reg_pattern(path)[0]
             if not regexp:
                 regexp = f"^{path}$"
-        _logger.debug(f"[path: {path}] map url regexp {regexp} to function: {filter_fun}")
+        _logger.debug(
+            f"[path: {path}] map url regexp {regexp} to function: {filter_fun}")
         self.filter_mapping[regexp] = filter_fun
 
     def get_matched_filters(self, path):
@@ -232,24 +235,47 @@ class RoutingConf:
         available_filters = []
         for regexp, val in self.filter_mapping.items():
             m = re.match(regexp, path)
-            _logger.debug(f"filter:: [{regexp}], path:: [{path}] match? {m is not None}")
+            _logger.debug(
+                f"filter:: [{regexp}], path:: [{path}] match? {m is not None}")
             if m:
                 available_filters.append(val)
         return available_filters
 
-    def map_websocket_handler(self, endpoint, handler_class):
-        url = remove_url_first_slash(endpoint)
-        path_pattern, path_names = get_path_reg_pattern(url)
-        if path_pattern is None:
-            self.ws_mapping[url] = handler_class
+    def map_websocket_handler(self, handler: WebsocketHandlerClass):
+        url = handler.url
+        regexp = handler.regexp
+        _logger.debug(
+            f"map url {url}|{regexp} to controller class {handler.cls}")
+        if regexp:
+            self.ws_regx_mapping[regexp] = handler
         else:
-            self.ws_path_val_mapping[path_pattern] = (
-                handler_class, path_names)
+            url = remove_url_first_slash(url)
+            path_pattern, path_names = get_path_reg_pattern(url)
+            if path_pattern is None:
+                self.ws_mapping[url] = handler
+            else:
+                self.ws_path_val_mapping[path_pattern] = (handler, path_names)
 
     def get_websocket_handler(self, path):
+        # explicitly mapping
         if path in self.ws_mapping:
-            return self.ws_mapping[path], {}
-        return self.__try_get_ws_handler_from_path_val(path)
+            return self.ws_mapping[path], {}, ()
+
+        # path value mapping
+        handler, path_vals = self.__try_get_ws_handler_from_path_val(path)
+        if handler is not None:
+            return handler, path_vals, ()
+        # regexp mapping
+        return self.__try_get_ws_hanlder_from_regexp(path)
+
+    def __try_get_ws_hanlder_from_regexp(self, path):
+        for regex, handler in self.ws_regx_mapping.items():
+            m = re.match(regex, f"/{path}") or re.match(regex, path)
+            _logger.debug(
+                f"regexp::pattern::[{regex}] => path::[{path}] match? {m is not None}")
+            if m:
+                return handler, {}, tuple([unquote(v) for v in m.groups()])
+        return None, {}, ()
 
     def __try_get_ws_handler_from_path_val(self, path):
         for patterns, val in self.ws_path_val_mapping.items():
@@ -257,12 +283,12 @@ class RoutingConf:
             _logger.debug(
                 f"websocket endpoint with path value::pattern::[{patterns}] => path::[{path}] match? {m is not None}")
             if m:
-                clz, path_names = val
+                handler, path_names = val
                 path_values = {}
                 for idx in range(len(path_names)):
                     key = unquote(path_names[idx])
                     path_values[key] = unquote(m.groups()[idx])
-                return clz, path_values
+                return handler, path_values
         return None, {}
 
     def map_error_page(self, code: str, error_page_fun: Callable):
@@ -350,8 +376,20 @@ class RoutingConf:
         else:
             return func()
 
+    @abstractmethod
+    def start(self):
+        pass
 
-class HTTPServer(TCPServer, RoutingConf):
+    @abstractmethod
+    async def start_async(self):
+        pass
+
+    @abstractmethod
+    def shutdown(self):
+        pass
+
+
+class HTTPServer(TCPServer, RoutingServer):
 
     allow_reuse_address = 1    # Seems to make sense in testing environment
 
@@ -366,7 +404,7 @@ class HTTPServer(TCPServer, RoutingConf):
 
     def __init__(self, addr, res_conf={}, max_workers: int = None):
         TCPServer.__init__(self, addr, SocketServerStreamRequestHandlerWraper)
-        RoutingConf.__init__(self, res_conf)
+        RoutingServer.__init__(self, res_conf)
         self.max_workers = max_workers or self._default_max_workers
         self.threadpool: ThreadPoolExecutor = ThreadPoolExecutor(
             thread_name_prefix="ReqThread",
@@ -382,7 +420,8 @@ class HTTPServer(TCPServer, RoutingConf):
 
     # override
     def process_request(self, request, client_address):
-        self.threadpool.submit(self.process_request_thread, request, client_address)
+        self.threadpool.submit(
+            self.process_request_thread, request, client_address)
 
     def server_close(self):
         super().server_close()
@@ -391,6 +430,9 @@ class HTTPServer(TCPServer, RoutingConf):
     def start(self):
         self.serve_forever()
 
+    async def start_async(self):
+        self.start()
+
     def _shutdown(self) -> None:
         _logger.debug("shutdown http server in a seperate thread..")
         super().shutdown()
@@ -399,10 +441,10 @@ class HTTPServer(TCPServer, RoutingConf):
         threading.Thread(target=self._shutdown, daemon=False).start()
 
 
-class CoroutineHTTPServer(RoutingConf):
+class CoroutineHTTPServer(RoutingServer):
 
     def __init__(self, host: str = '', port: int = 9090, ssl: SSLContext = None, res_conf={}) -> None:
-        RoutingConf.__init__(self, res_conf)
+        RoutingServer.__init__(self, res_conf)
         self.host: str = host
         self.port: int = port
         self.ssl: SSLContext = ssl
@@ -414,7 +456,7 @@ class CoroutineHTTPServer(RoutingConf):
         _logger.debug("Connection ends, close the writer.")
         writer.close()
 
-    async def start_server(self):
+    async def start_async(self):
         self.server = await asyncio.start_server(
             self.callback, host=self.host, port=self.port, ssl=self.ssl)
         async with self.server:
@@ -427,7 +469,7 @@ class CoroutineHTTPServer(RoutingConf):
                 await self.server.wait_closed()
 
     def start(self):
-        asyncio.run(self.start_server())
+        asyncio.run(self.start_async())
 
     def _shutdown(self):
         _logger.debug("Try to shutdown server.")
@@ -455,8 +497,8 @@ class SimpleDispatcherHttpServer:
     def map_controller(self, ctrl: ControllerFunction):
         self.server.map_controller(ctrl)
 
-    def map_websocket_handler(self, endpoint, handler_class):
-        self.server.map_websocket_handler(endpoint, handler_class)
+    def map_websocket_handler(self, handler: WebsocketHandlerClass):
+        self.server.map_websocket_handler(handler)
 
     def map_error_page(self, code, func):
         self.server.map_error_page(code, func)
@@ -492,12 +534,15 @@ class SimpleDispatcherHttpServer:
             self.ssl_ctx = None
 
         if prefer_corountine:
-            _logger.info(f"Start server in corouting mode, listen to port: {self.host[1]}")
+            _logger.info(
+                f"Start server in corouting mode, listen to port: {self.host[1]}")
             self.server = CoroutineHTTPServer(
                 self.host[0], self.host[1], self.ssl_ctx, resources)
         else:
-            _logger.info(f"Start server in threading mixed mode, listen to port {self.host[1]}")
-            self.server = HTTPServer(self.host, resources, max_workers=max_workers)
+            _logger.info(
+                f"Start server in threading mixed mode, listen to port {self.host[1]}")
+            self.server = HTTPServer(
+                self.host, resources, max_workers=max_workers)
             if self.ssl_ctx:
                 self.server.socket = self.ssl_ctx.wrap_socket(
                     self.server.socket, server_side=True)
@@ -517,12 +562,20 @@ class SimpleDispatcherHttpServer:
             self.__ready = False
             raise
 
+    async def start_async(self):
+        try:
+            self.__ready = True
+            await self.server.start_async()
+        except:
+            self.__ready = False
+            raise
+
     def shutdown(self):
         # shutdown it in a seperate thread.
         self.server.shutdown()
 
 
-class WSGIProxy(RoutingConf):
+class WSGIProxy(RoutingServer):
 
     def __init__(self, res_conf):
         super().__init__(res_conf=res_conf)
