@@ -41,7 +41,7 @@ from simple_http_server.logger import get_logger
 
 __logger = get_logger("simple_http_server.server")
 __lock = threading.Lock()
-_server = None
+_server: http_server.RoutingServer = None
 
 
 def _is_match(string="", regx=r""):
@@ -102,17 +102,17 @@ def scan(base_dir: str = "", regx: str = r"", project_dir: str = "") -> None:
         _import_module(mname)
 
 
-def start(host: str = "",
-          port: int = 9090,
-          ssl: bool = False,
-          ssl_protocol: int = PROTOCOL_TLS_SERVER,
-          ssl_check_hostname: bool = False,
-          keyfile: str = "",
-          certfile: str = "",
-          keypass: str = "",
-          ssl_context: SSLContext = None,
-          resources: Dict[str, str] = {},
-          prefer_coroutine=False) -> None:
+def _prepare_server(host: str = "",
+                    port: int = 9090,
+                    ssl: bool = False,
+                    ssl_protocol: int = PROTOCOL_TLS_SERVER,
+                    ssl_check_hostname: bool = False,
+                    keyfile: str = "",
+                    certfile: str = "",
+                    keypass: str = "",
+                    ssl_context: SSLContext = None,
+                    resources: Dict[str, str] = {},
+                    prefer_coroutine=False) -> None:
     with __lock:
         global _server
         if _server is not None:
@@ -142,17 +142,72 @@ def start(host: str = "",
 
     ws_handlers = _get_websocket_handlers()
 
-    for endpoint, clz in ws_handlers.items():
-        _server.map_websocket_handler(endpoint, clz)
+    for wshandler in ws_handlers:
+        _server.map_websocket_handler(wshandler)
 
     err_pages = _get_error_pages()
     for code, func in err_pages.items():
         _server.map_error_page(code, func)
 
+
+def start(host: str = "",
+          port: int = 9090,
+          ssl: bool = False,
+          ssl_protocol: int = PROTOCOL_TLS_SERVER,
+          ssl_check_hostname: bool = False,
+          keyfile: str = "",
+          certfile: str = "",
+          keypass: str = "",
+          ssl_context: SSLContext = None,
+          resources: Dict[str, str] = {},
+          prefer_coroutine=False) -> None:
+    _prepare_server(
+        host=host,
+        port=port,
+        ssl=ssl,
+        ssl_protocol=ssl_protocol,
+        ssl_check_hostname=ssl_check_hostname,
+        keyfile=keyfile,
+        certfile=certfile,
+        keypass=keypass,
+        ssl_context=ssl_context,
+        resources=resources,
+        prefer_coroutine=prefer_coroutine
+    )
+
     # start the server
     _server.start()
 
 
+async def start_async(host: str = "",
+                      port: int = 9090,
+                      ssl: bool = False,
+                      ssl_protocol: int = PROTOCOL_TLS_SERVER,
+                      ssl_check_hostname: bool = False,
+                      keyfile: str = "",
+                      certfile: str = "",
+                      keypass: str = "",
+                      ssl_context: SSLContext = None,
+                      resources: Dict[str, str] = {},
+                      prefer_coroutine=False) -> None:
+    _prepare_server(
+        host=host,
+        port=port,
+        ssl=ssl,
+        ssl_protocol=ssl_protocol,
+        ssl_check_hostname=ssl_check_hostname,
+        keyfile=keyfile,
+        certfile=certfile,
+        keypass=keypass,
+        ssl_context=ssl_context,
+        resources=resources,
+        prefer_coroutine=prefer_coroutine
+    )
+
+    # start the server
+    await _server.start_async()
+
+
 def is_ready() -> bool:
     return _server and _server.ready
 
@@ -284,8 +339,8 @@ def init_wsgi_proxy(resources: Dict[str, str] = {}, session_factory=None) -> htt
 
     ws_handlers = _get_websocket_handlers()
 
-    for endpoint, clz in ws_handlers.items():
-        proxy.map_websocket_handler(endpoint, clz)
+    for hander in ws_handlers:
+        proxy.map_websocket_handler(hander)
 
     err_pages = _get_error_pages()
     for code, func in err_pages.items():
